@@ -7,7 +7,11 @@
  *
  */
 
-#include "fte.h"
+#include "c_bind.h"
+#include "s_string.h"
+#include "sysdep.h"
+
+#include <ctype.h>
 
 //////////////////////////////////////////////////////////////////////////////
 
@@ -23,6 +27,122 @@ ExMacro *Macros = 0;
 
 //////////////////////////////////////////////////////////////////////////////
 
+static const struct {
+    const char Name[8];
+    TKeyCode Key;
+} KeyList[] = {
+    { "Esc", kbEsc },
+    { "Tab", kbTab },
+    { "Space", kbSpace },
+    { "Enter", kbEnter },
+    { "BackSp", kbBackSp },
+    { "F1", kbF1 },
+    { "F2", kbF2 },
+    { "F3", kbF3 },
+    { "F4", kbF4 },
+    { "F5", kbF5 },
+    { "F6", kbF6 },
+    { "F7", kbF7 },
+    { "F8", kbF8 },
+    { "F9", kbF9 },
+    { "F10", kbF10 },
+    { "F11", kbF11 },
+    { "F12", kbF12 },
+    { "Left", kbLeft },
+    { "Right", kbRight },
+    { "Up", kbUp },
+    { "Down", kbDown },
+    { "Home", kbHome },
+    { "End", kbEnd },
+    { "PgUp", kbPgUp },
+    { "PgDn", kbPgDn },
+    { "Ins", kbIns },
+    { "Del", kbDel },
+    { "Center", kbCenter },
+    { "Break", kbBreak },
+    { "Pause", kbPause },
+    { "PrtScr", kbPrtScr },
+    { "SysReq", kbSysReq },
+};
+
+static int ParseKey(const char *Key, KeySel &ks) {
+    TKeyCode KeyFlags = 0;
+
+    ks.Mask = 0;
+    ks.Key = 0;
+    while (Key[0] && ((Key[1] == '+') || (Key[1] == '-'))) {
+        if (Key[1] == '-') {
+            switch (Key[0]) {
+            case 'A': ks.Mask |= kfAlt; break;
+            case 'C': ks.Mask |= kfCtrl; break;
+            case 'S': ks.Mask |= kfShift; break;
+            case 'G': ks.Mask |= kfGray; break;
+            case 'X': ks.Mask |= kfSpecial; break;
+            }
+        } else if (Key[1] == '+') {
+            switch (Key[0]) {
+            case 'A': KeyFlags |= kfAlt; break;
+            case 'C': KeyFlags |= kfCtrl; break;
+            case 'S': KeyFlags |= kfShift; break;
+            case 'G': KeyFlags |= kfGray; break;
+            case 'X': KeyFlags |= kfSpecial; break;
+            }
+        }
+        Key += 2;
+    }
+
+    for (size_t i = 0; i < FTE_ARRAY_SIZE(KeyList); ++i)
+        if (strcmp(Key, KeyList[i].Name) == 0) {
+            ks.Key = KeyList[i].Key;
+            break;
+        }
+
+    if (ks.Key == 0)
+        ks.Key = *Key;
+    if ((KeyFlags & kfCtrl) && !(KeyFlags & kfSpecial)) {
+        if (ks.Key < 256) {
+            if (ks.Key < 32)
+                ks.Key += 64;
+            else
+                ks.Key = toupper(ks.Key);
+        }
+    }
+    ks.Key |= KeyFlags;
+    return 0;
+}
+
+int GetKeyName(char *Key, size_t KeySize, KeySel &ks) {
+    strlcpy(Key, "", KeySize);
+
+    if (ks.Key  & kfAlt)   strlcat(Key, "A+", KeySize);
+    if (ks.Mask & kfAlt)   strlcat(Key, "A-", KeySize);
+    if (ks.Key  & kfCtrl)  strlcat(Key, "C+", KeySize);
+    if (ks.Mask & kfCtrl)  strlcat(Key, "C-", KeySize);
+    if (ks.Key  & kfGray)  strlcat(Key, "G+", KeySize);
+    if (ks.Mask & kfGray)  strlcat(Key, "G-", KeySize);
+    if (ks.Key  & kfShift) strlcat(Key, "S+", KeySize);
+    if (ks.Mask & kfShift) strlcat(Key, "S-", KeySize);
+
+    if (keyCode(ks.Key) < 256) {
+	char c[2] = { (char)ks.Key, 0 };
+
+        //if (ks.Key & kfCtrl)
+        //    if (c[0] < ' ')
+        //        c[0] += '@';
+        if (c[0] == 32)
+            strlcat(Key, "Space", KeySize);
+        else
+            strlcat(Key, c, KeySize);
+    } else {
+        for (size_t i = 0; i < FTE_ARRAY_SIZE(KeyList); ++i)
+            if (KeyList[i].Key == keyCode(ks.Key)) {
+                strlcat(Key, KeyList[i].Name, KeySize);
+                break;
+            }
+    }
+    return 0;
+}
+
 const char *GetCommandName(int Command) {
     if (Command & CMD_EXT) {
         Command &= ~CMD_EXT;
@@ -34,21 +154,17 @@ const char *GetCommandName(int Command) {
         else
             return "?NONE?";
     }
-    for (int i = 0; i < int(sizeof(Command_Table) / sizeof(Command_Table[0])); i++)
+    for (size_t i = 0; i < FTE_ARRAY_SIZE(Command_Table); ++i)
         if (Command_Table[i].CmdId == Command)
             return Command_Table[i].Name;
     return "?invalid?";
 }
 
-int CmdNum(char *Cmd) {
-    int i;
-
-    for (i = 0;
-         i < int(sizeof(Command_Table) / sizeof(Command_Table[0]));
-         i++)
+int CmdNum(const char *Cmd) {
+    for (size_t i = 0; i < FTE_ARRAY_SIZE(Command_Table); ++i)
         if (strcmp(Cmd, Command_Table[i].Name) == 0)
             return Command_Table[i].CmdId;
-    for (i = 0; i < CMacros; i++)
+    for (int i = 0; i < CMacros; i++)
         if (Macros[i].Name && (strcmp(Cmd, Macros[i].Name)) == 0)
             return i | CMD_EXT;
     return 0; // Nop
@@ -94,7 +210,7 @@ EKey *SetKey(EEventMap *aMap, const char *aKey) {
     char *p, *d;
     EEventMap *xm = aMap;
 
-    //    printf("Setting key %s\n", Key);
+    // printf("Setting key %s\n", Key);
     strcpy(Key, aKey);
 
     // if mode has parent, get parent keymap
@@ -176,15 +292,21 @@ EKey *SetKey(EEventMap *aMap, const char *aKey) {
     return 0;
 }
 
-void InitWordChars() {
+static void InitWordChars() {
     static int init = 0;
     if (init == 0) {
-        for (int i = 0; i < 256; i++)
-            if ((i >= '0' && i <= '9') ||
-                (i >= 'A' && i <= 'Z') ||
-                (i >= 'a' && i <= 'z') || (i == '_')) {
+	for (int i = 0; i < 256; i++)
+            // isalnum???
+	    //if (isdigit(i) || isalpha(i)
+	    //    || (i >= 'A' && i <= 'Z')
+            //    || (i >= 'a' && i <= 'z') || (i == '_')) {
+            if (isalnum(i) || (i == '_')) {
                 WSETBIT(DefaultBufferFlags.WordChars, i, 1);
-                if ((i >= 'A' && i <= 'Z'))
+                // Can someone tell me why we check A through Z?
+                // This won't work should someone port to EBCDIC (why, though?)
+                // besides, isupper is usually a #define that will compile to something
+                // even faster.
+                if (/*(i >= 'A' && i <= 'Z') || */ isupper(i))
                     WSETBIT(DefaultBufferFlags.CapitalChars, i, 1);
             }
         init = 1;
@@ -192,10 +314,9 @@ void InitWordChars() {
 }
 
 void SetWordChars(char *w, const char *s) {
-    const char *p;
-    memset((void *)w, 0, 32);
+    const char *p = s;
+    memset(w, 0, 32);
 
-    p = s;
     while (p && *p) {
         if (*p == '\\') {
             p++;
@@ -211,17 +332,26 @@ void SetWordChars(char *w, const char *s) {
     }
 }
 
-EMode::EMode(EMode *aMode, EEventMap *Map, const char *aName) {
-    fNext = 0;
-    fName = strdup(aName);
-    fEventMap = Map;
-    fParent = aMode;
+EMode::EMode(EMode *aMode, EEventMap *Map, const char *aName) :
+    fNext(0),
+    fName(strdup(aName)),
+    fEventMap(Map),
+    fParent(aMode)
+{
     InitWordChars();
     if (aMode) {
 #ifdef CONFIG_SYNTAX_HILIT
         fColorize = aMode->fColorize;
 #endif
         Flags = aMode->Flags;
+
+        // duplicate strings in flags to allow them be freed
+        for (int i=0; i<BFS_COUNT; i++)
+        {
+            if (aMode->Flags.str[i] != 0)
+                Flags.str[i] = strdup(aMode->Flags.str[i]);
+        }
+
         MatchName = 0;
         MatchLine = 0;
         MatchNameRx = 0;
@@ -243,12 +373,44 @@ EMode::EMode(EMode *aMode, EEventMap *Map, const char *aName) {
         fColorize = 0;
 #endif
         Flags = DefaultBufferFlags;
+
+        // there is no strings in default settings...
     }
 }
 
-EKeyMap::EKeyMap() {
-    fKeys = 0;
-    fParent = 0;
+EMode::~EMode() {
+
+    // fEventMap is just pointer to EventMaps list, so do not destroy it
+    // fColorize is also just a pointer
+
+    free(fName);
+
+    free(MatchName);
+    RxFree(MatchNameRx);
+
+    free(MatchLine);
+    RxFree(MatchLineRx);
+
+    // free strings from flags
+    for (unsigned i = 0; i < BFS_COUNT; i++)
+        free(Flags.str[i]);
+}
+
+EKeyMap::EKeyMap() :
+    fParent(NULL),
+    fKeys(NULL)
+{
+}
+
+EKeyMap::~EKeyMap() {
+    // free keys
+    EKey *e;
+
+    while((e = fKeys) != NULL)
+    {
+	fKeys = fKeys->fNext;
+	delete e;
+    }
 }
 
 void EKeyMap::AddKey(EKey *aKey) {
@@ -257,18 +419,19 @@ void EKeyMap::AddKey(EKey *aKey) {
 }
 
 
-int MatchKey(TKeyCode aKey, KeySel aSel) {
+static int MatchKey(TKeyCode aKey, KeySel aSel) {
     long flags = aKey & ~ 0xFFFF;
     long key = aKey & 0xFFFF;
 
     flags &= ~kfAltXXX;
 
     if (flags & kfShift) {
-        if (key < 256)
+        if (key < 256) {
             if (flags == kfShift)
                 flags &= ~kfShift;
             else if (isascii(key))
                 key = toupper(key);
+        }
     }
     if ((flags & kfCtrl) && !(flags & kfSpecial))
         if (key < 32)
@@ -303,11 +466,30 @@ EEventMap::EEventMap(const char *AName, EEventMap *AParent) {
     KeyMap = 0;
     Next = EventMaps;
     EventMaps = this;
-    Menu[0] = Menu[1] = 0;
+    memset(Menu, 0, sizeof(Menu));
     memset(abbrev, 0, sizeof(abbrev));
 }
 
 EEventMap::~EEventMap() {
+    free(Name);
+
+    // free menu[]
+    for (int i = 0; i < EM_MENUS; i++)
+	free(Menu[i]);
+
+#ifdef CONFIG_ABBREV
+    // free Abbrev's
+    EAbbrev *ab;
+
+    for (int i = 0; i < ABBREV_HASH; i++)
+	while((ab = abbrev[i]) != NULL)
+	{
+	    abbrev[i] = abbrev[i]->next;
+	    delete ab;
+	}
+#endif
+    // free keymap's
+    delete KeyMap;
 }
 
 void EEventMap::SetMenu(int which, const char *What) {
@@ -356,159 +538,48 @@ EAbbrev *EMode::FindAbbrev(const char *string) {
 }
 #endif
 
-static const struct {
-    const char *Name;
-    TKeyCode Key;
-} KeyList[] = {
-    { "Esc", kbEsc },
-    { "Tab", kbTab },
-    { "Space", kbSpace },
-    { "Enter", kbEnter },
-    { "BackSp", kbBackSp },
-    { "F1", kbF1 },
-    { "F2", kbF2 },
-    { "F3", kbF3 },
-    { "F4", kbF4 },
-    { "F5", kbF5 },
-    { "F6", kbF6 },
-    { "F7", kbF7 },
-    { "F8", kbF8 },
-    { "F9", kbF9 },
-    { "F10", kbF10 },
-    { "F11", kbF11 },
-    { "F12", kbF12 },
-    { "Left", kbLeft },
-    { "Right", kbRight },
-    { "Up", kbUp },
-    { "Down", kbDown },
-    { "Home", kbHome },
-    { "End", kbEnd },
-    { "PgUp", kbPgUp },
-    { "PgDn", kbPgDn },
-    { "Ins", kbIns },
-    { "Del", kbDel },
-    { "Center", kbCenter },
-    { "Break", kbBreak },
-    { "Pause", kbPause },
-    { "PrtScr", kbPrtScr },
-    { "SysReq", kbSysReq },
-};
-
-int ParseKey(const char *Key, KeySel &ks) {
-    unsigned char *p = (unsigned char *)Key;
-    TKeyCode KeyFlags = 0;
-    int i;
-
-    ks.Mask = 0;
-    ks.Key = 0;
-    while ((*p) && ((p[1] == '+') || (p[1] == '-'))) {
-        if (p[1] == '-') {
-            switch (p[0]) {
-            case 'A': ks.Mask |= kfAlt; break;
-            case 'C': ks.Mask |= kfCtrl; break;
-            case 'S': ks.Mask |= kfShift; break;
-            case 'G': ks.Mask |= kfGray; break;
-            case 'X': ks.Mask |= kfSpecial; break;
-            }
-        } else if (p[1] == '+') {
-            switch (p[0]) {
-            case 'A': KeyFlags |= kfAlt; break;
-            case 'C': KeyFlags |= kfCtrl; break;
-            case 'S': KeyFlags |= kfShift; break;
-            case 'G': KeyFlags |= kfGray; break;
-            case 'X': KeyFlags |= kfSpecial; break;
-            }
-        }
-        p += 2;
-    }
-    for (i = 0; i < int(sizeof(KeyList)/sizeof(KeyList[0])); i++)
-        if (strcmp((char *)p, KeyList[i].Name) == 0) {
-            ks.Key = KeyList[i].Key;
-            break;
-        }
-    if (ks.Key == 0)
-        ks.Key = *p;
-    if ((KeyFlags & kfCtrl) && !(KeyFlags & kfSpecial)) {
-        if (ks.Key < 256) {
-            if (ks.Key < 32)
-                ks.Key += 64;
-            else
-                ks.Key = toupper(ks.Key);
-        }
-    }
-    ks.Key |= KeyFlags;
-    return 0;
-}
-
-int GetKeyName(char *Key, KeySel &ks) {
-    *Key = 0;
-
-    if (ks.Key  & kfAlt)   strcat(Key, "A+");
-    if (ks.Mask & kfAlt)   strcat(Key, "A-");
-    if (ks.Key  & kfCtrl)  strcat(Key, "C+");
-    if (ks.Mask & kfCtrl)  strcat(Key, "C-");
-    if (ks.Key  & kfGray)  strcat(Key, "G+");
-    if (ks.Mask & kfGray)  strcat(Key, "G-");
-    if (ks.Key  & kfShift) strcat(Key, "S+");
-    if (ks.Mask & kfShift) strcat(Key, "S-");
-
-    if (keyCode(ks.Key) < 256) {
-        char c[2];
-
-        c[0] = (char)(ks.Key & 0xFF);
-        c[1] = 0;
-
-        //if (ks.Key & kfCtrl)
-        //    if (c[0] < ' ')
-        //        c[0] += '@';
-        if (c[0] == 32)
-            strcat(Key, "Space");
-        else
-            strcat(Key, c);
-    } else {
-        for (int i = 0; i < int(sizeof(KeyList)/sizeof(KeyList[0])); i++)
-            if (KeyList[i].Key == keyCode(ks.Key)) {
-                strcat(Key, KeyList[i].Name);
-                break;
-            }
-    }
-    return 0;
-}
-
-EKey::EKey(char *aKey) {
-    fNext = 0;
+EKey::EKey(const char *aKey) :
+    Cmd(-1),
+    fKeyMap(0),
+    fNext(0)
+{
     ParseKey(aKey, fKey);
-    fKeyMap = 0;
-    Cmd = -1;
 }
 
-EKey::EKey(char *aKey, EKeyMap *aKeyMap) {
-    fNext = 0;
-    Cmd = -1;
+EKey::EKey(const char *aKey, EKeyMap *aKeyMap) :
+    Cmd(-1),
+    fKeyMap(aKeyMap),
+    fNext(0)
+{
     ParseKey(aKey, fKey);
-    fKeyMap = aKeyMap;
+}
+
+EKey::~EKey()
+{
+    // if there is child keymaps delete them
+    delete fKeyMap;
 }
 
 #ifdef CONFIG_ABBREV
-EAbbrev::EAbbrev(const char *aMatch, const char *aReplace) {
-    next = 0;
-    Match = strdup(aMatch);
-    Replace = strdup(aReplace);
-    Cmd = -1;
+EAbbrev::EAbbrev(const char *aMatch, const char *aReplace) :
+    next(0),
+    Cmd(-1),
+    Match(strdup(aMatch)),
+    Replace(strdup(aReplace))
+{
 }
 
-EAbbrev::EAbbrev(const char *aMatch, int aCmd) {
-    next = 0;
-    Replace = 0;
-    Match = strdup(aMatch);
-    Cmd = aCmd;
+EAbbrev::EAbbrev(const char *aMatch, int aCmd) :
+    next(0),
+    Cmd(aCmd),
+    Match(strdup(aMatch)),
+    Replace(0)
+{
 }
 
 EAbbrev::~EAbbrev() {
-    if (Match)
-        free(Match);
-    if (Replace)
-        free(Replace);
+    free(Match);
+    free(Replace);
 }
 #endif
 
@@ -568,20 +639,19 @@ int NewCommand(const char *Name) {
     Macros = (ExMacro *) realloc(Macros, sizeof(ExMacro) * (1 + CMacros));
     Macros[CMacros].Count = 0;
     Macros[CMacros].cmds = 0;
-    Macros[CMacros].Name = Name ? strdup(Name) : 0;
+    Macros[CMacros].Name = (Name != NULL) ? strdup(Name) : 0;
     CMacros++;
     return CMacros - 1;
 }
 
-int ExState::GetStrParam(EView *view, char *str, int maxlen) {
-    assert(maxlen >= 0);
-    if (Macro == -1 || Pos == -1) return 0;
-    if (Pos >= Macros[Macro].Count) return 0;
+int ExState::GetStrParam(EView *view, char *str, size_t maxlen) {
+    if (Macro == -1
+	|| Pos == -1
+	|| Pos >= Macros[Macro].Count)
+	return 0;
     if (Macros[Macro].cmds[Pos].type == CT_STRING) {
-        if (maxlen > 0) {
-            strncpy(str, Macros[Macro].cmds[Pos].u.string, maxlen);
-            str[maxlen - 1] = 0;
-        }
+        if (maxlen > 0)
+            strlcpy(str, Macros[Macro].cmds[Pos].u.string, maxlen);
         Pos++;
     } else if (view && Macros[Macro].cmds[Pos].type == CT_VARIABLE) {
         //puts("variable\x7");
@@ -593,10 +663,11 @@ int ExState::GetStrParam(EView *view, char *str, int maxlen) {
     if (Pos < Macros[Macro].Count) {
         if (Macros[Macro].cmds[Pos].type == CT_CONCAT) {
             Pos++;
-            int len = strlen(str);
-            int left = maxlen - len;
+            size_t len = strlen(str);
 
-            assert(left >= 0);
+            if (len >= maxlen)
+                return 0;
+            size_t left = maxlen - len;
 
             //puts("concat\x7");
             if (GetStrParam(view, str + len, left) == 0)
@@ -607,8 +678,10 @@ int ExState::GetStrParam(EView *view, char *str, int maxlen) {
 }
 
 int ExState::GetIntParam(EView *view, int *value) {
-    if (Macro == -1 || Pos == -1) return 0;
-    if (Pos >= Macros[Macro].Count) return 0;
+    if (Macro == -1
+	|| Pos == -1
+	|| Pos >= Macros[Macro].Count)
+	return 0;
     if (Macros[Macro].cmds[Pos].type == CT_NUMBER) {
         *value = Macros[Macro].cmds[Pos].u.num;
         Pos++;
@@ -621,12 +694,11 @@ int ExState::GetIntParam(EView *view, int *value) {
     return 1;
 }
 
-int HashStr(const char *p, int max) {
+int HashStr(const char *p, int maxim) {
     unsigned int i = 1;
 
-    while (p && *p) {
-        i += i ^ (i << 3) ^ (unsigned int)(*p) ^ (i >> 3);
-        p++;
-    }
-    return i % max;
+    while (p && *p)
+	i += i ^ (i << 3) ^ (unsigned int)(*p++) ^ (i >> 3);
+
+    return i % maxim;
 }
