@@ -7,11 +7,16 @@
  *
  */
 
-#include "fte.h"
+#include "o_buflist.h"
 
 #ifdef CONFIG_SYNTAX_HILIT
+
+#include "sysdep.h"
+
+#include <stdio.h>
+
 static const struct {
-    const char *Name;
+    const char Name[8];
     int Num;
     SyntaxProc Proc;
 } HilitModes[] = {
@@ -58,7 +63,7 @@ static const struct {
 };
 
 static const struct {
-    const char *Name;
+    const char Name[8];
     int Num;
 } IndentModes[] = {
 #ifdef CONFIG_INDENT_C
@@ -76,21 +81,21 @@ static const struct {
 EColorize *Colorizers = 0;
 
 int GetIndentMode(const char *Str) {
-    for (unsigned int i = 0; i < sizeof(IndentModes) / sizeof(IndentModes[0]); i++)
+    for (size_t i = 0; i < FTE_ARRAY_SIZE(IndentModes); ++i)
         if (strcmp(Str, IndentModes[i].Name) == 0)
             return IndentModes[i].Num;
     return 0;
 }
 
 int GetHilitMode(const char *Str) {
-    for (unsigned int i = 0; i < sizeof(HilitModes) / sizeof(HilitModes[0]); i++)
+    for (size_t i = 0; i < FTE_ARRAY_SIZE(HilitModes); ++i)
         if (strcmp(Str, HilitModes[i].Name) == 0)
             return HilitModes[i].Num;
     return HILIT_PLAIN;
 }
 
 SyntaxProc GetHilitProc(int id) {
-    for (unsigned int i = 0; i < sizeof(HilitModes) / sizeof(HilitModes[0]); i++)
+    for (size_t i = 0; i < FTE_ARRAY_SIZE(HilitModes); ++i)
         if (id == HilitModes[i].Num)
             return HilitModes[i].Proc;
     return 0;
@@ -100,35 +105,30 @@ SyntaxProc GetHilitProc(int id) {
 int EBuffer::HilitAddWord(const char *Word) {
     if (HilitFindWord(Word) == 1)
 	return 1;
-    WordList = (char **)realloc((void *)WordList, (1 + WordCount) * sizeof(char *));
-    if (WordList == 0) return 0;
-    WordList[WordCount++] = strdup(Word);
+    WordList.push_back(Word);
     FullRedraw();
     return 1;
 }
 
 int EBuffer::HilitFindWord(const char *Word) {
-    for (int i = 0; i < WordCount; i++) {
+    vector_iterate(StlString, WordList, it) {
         if (BFI(this, BFI_MatchCase) == 1) {
-            if (strcmp(Word, WordList[i]) == 0) return 1;
+            if (strcmp(Word, (*it).c_str()) == 0) return 1;
         } else {
-            if (stricmp(Word, WordList[i]) == 0) return 1;
+            if (stricmp(Word, (*it).c_str()) == 0) return 1;
         }
     }
     return 0;
 }
 
 int EBuffer::HilitRemoveWord(const char *Word) {
-    for (int i = 0; i < WordCount; i++) {
+    vector_iterate(StlString, WordList, it) {
         if (BFI(this, BFI_MatchCase) == 1) {
-            if (strcmp(Word, WordList[i]) != 0) continue;
+            if (strcmp(Word, (*it).c_str()) != 0) continue;
         } else {
-            if (stricmp(Word, WordList[i]) != 0) continue;
-        }
-        free(WordList[i]);
-        memmove(WordList + i, WordList + i + 1, sizeof(char *) * (WordCount - i - 1));
-        WordCount--;
-        WordList = (char **)realloc((void *)WordList, WordCount * sizeof(char *));
+            if (stricmp(Word, (*it).c_str()) != 0) continue;
+	}
+        WordList.erase(it);
         FullRedraw();
         return 1;
     }
@@ -143,7 +143,7 @@ int EBuffer::HilitWord() {
     P = CharOffset(L, CP.Col);
     while ((P > 0) && ((ChClass(L->Chars[P - 1]) == 1) || (L->Chars[P - 1] == '_'))) 
         P--;
-    while (len < CK_MAXLEN && P < L->Count && (ChClass(L->Chars[P]) == 1 || L->Chars[P] == '_'))
+    while (len < CK_MAXLEN && P < (int)L->Count && (ChClass(L->Chars[P]) == 1 || L->Chars[P] == '_'))
         s[len++] = L->Chars[P++];
     if (len == 0)
         return 0;
@@ -155,26 +155,23 @@ int EBuffer::HilitWord() {
 
 /* ======================================================================= */
 
-EColorize::EColorize(const char *AName, const char *AParent) {
-    Name = strdup(AName);
-    SyntaxParser = HILIT_PLAIN;
-    Next = Colorizers;
-    hm = 0;
+EColorize::EColorize(const char *AName, const char *AParent) :
+    Name(AName),
+    Next(Colorizers),
+    Parent(FindColorizer(AParent)),
+    SyntaxParser(HILIT_PLAIN),
+    hm(0)
+{
     Colorizers = this;
-    Parent = FindColorizer(AParent);
-    memset((void *)&Keywords, 0, sizeof(Keywords));
-    memset((void *)Colors, 0, sizeof(Colors));
+    memset(&Keywords, 0, sizeof(Keywords));
     if (Parent) {
         SyntaxParser = Parent->SyntaxParser;
-        memcpy((void *)Colors, (void *)Parent->Colors, sizeof(Colors));
-    } else {
-        SyntaxParser = HILIT_PLAIN;
-    }
+        memcpy(Colors, Parent->Colors, sizeof(Colors));
+    } else
+	memset(Colors, 0, sizeof(Colors));
 }
 
 EColorize::~EColorize() {
-    free(Name);
-
     for (int i=0; i<CK_MAXLEN; i++)
         free(Keywords.key[i]);
 
@@ -185,7 +182,7 @@ EColorize *FindColorizer(const char *AName) {
     EColorize *p = Colorizers;
 
     while (p) {
-        if (strcmp(AName, p->Name) == 0)
+        if (p->Name == AName)
             return p;
         p = p->Next;
     }
@@ -200,7 +197,7 @@ int EColorize::SetColor(int idx, const char *Value) {
 
     if (idx < 0 || idx >= COUNT_CLR)
         return 0;
-    Colors[idx] = ColFg | (ColBg << 4);
+    Colors[idx] = ChColor(ColFg | (ColBg << 4));
     return 1;
 }
 
@@ -229,10 +226,10 @@ void HState::InitState() {
     nextKwdNoCharState = -1;
 }
 
-int HState::GetHilitWord(int len, char *str, ChColor &clr) {
+int HState::GetHilitWord(ChColor &clr, const char *str, size_t len) {
     char *p;
 
-    if (len >= CK_MAXLEN || len < 1)
+    if (len >= CK_MAXLEN || !len)
         return 0;
 
     p = keywords.key[len];
@@ -258,11 +255,12 @@ int HState::GetHilitWord(int len, char *str, ChColor &clr) {
 
 /* ======================================================================= */
 
-HMachine::HMachine() {
-    stateCount = 0;
-    state = 0;
-    transCount = 0;
-    trans = 0;
+HMachine::HMachine() :
+    stateCount(0),
+    transCount(0),
+    state(0),
+    trans(0)
+{
 }
 
 HMachine::~HMachine() {
@@ -270,11 +268,9 @@ HMachine::~HMachine() {
     // free states
     if (state)
     {
-        int i;
-
         while(stateCount--)
         {
-            for (i=0; i<CK_MAXLEN; i++)
+            for (unsigned i = 0; i < CK_MAXLEN; ++i)
                 free(state[stateCount].keywords.key[i]);
 
             free(state[stateCount].wordChars);
@@ -314,4 +310,4 @@ void HMachine::AddTrans(HTrans &aTrans) {
     transCount++;
 }
 
-#endif
+#endif // CONFIG_SYNTAX_HILIT
