@@ -7,8 +7,10 @@
  *
  */
 
-#ifndef __REGEX_H
-#define __REGEX_H
+#ifndef E_REGEX_H
+#define E_REGEX_H
+
+#include <sys/types.h>
 
 /*
  * Operator:
@@ -32,10 +34,14 @@
  * \x##         Match character with ASCII code ## (hex)
  * \N###        Match ascii code ### (dec)
  * \o###        Match ascii code
- * \a           Match \a              \r           Match 0x13 (cr)
- * \b           Match \b              \t           Match 0x09 (tab)
- * \f           Match \f              \v           Match \v
- * \n           Match 0x10 (lf)       \e           Match escape (^E)
+ * \a           Match \a
+ * \b           Match \b
+ * \f           Match \f
+ * \n           Match 0x0a (lf)
+ * \r           Match 0x0d (cr)
+ * \t           Match 0x09 (tab)
+ * \v           Match \v
+ * \e           Match escape (^E)
  * \s           Match whitespace (cr/lf/tab/space)
  * \S           Match nonwhitespace (!\S)
  * \w           Match word character
@@ -47,7 +53,7 @@
  * \C           Match case sensitively from here on
  * \c           Match case ingnore from here on
  */
-
+// *INDENT-OFF*
 #define RE_NOTHING         0  // nothing
 #define RE_JUMP            1  // jump to
 #define RE_BREAK           2  // break |
@@ -82,11 +88,10 @@
 
 #define NSEXPS            64  // for replace only 0-9
 
-#define RX_CASE         1  // matchcase
+#define RX_CASE            1  // matchcase
+// *INDENT-ON*
 
-typedef struct _RxNode RxNode;
-
-struct _RxNode {
+struct RxNode {
     short fWhat;
     short fLen;
     RxNode *fPrev;
@@ -97,14 +102,15 @@ struct _RxNode {
     };
 };
 
-typedef struct {
+struct RxMatchRes {
     int Open[NSEXPS];    // -1 = not matched
     int Close[NSEXPS];
-} RxMatchRes;
+};
 
 RxNode *RxCompile(const char *Regexp);
-int RxExec(RxNode *Regexp, const char *Data, int Len, const char *Start, RxMatchRes *Match, unsigned int RxOpt = RX_CASE);
-int RxReplace(const char *rep, const char *Src, int len, RxMatchRes match, char **Dest, int *Dlen);
+int RxExecMatch(RxNode *Regexp, const char *Data, size_t Len, const char *Start, RxMatchRes *Match, unsigned int RxOpt = RX_CASE);
+int RxExec(RxNode *Regexp, const char *Data, size_t Len, const char *Start, RxMatchRes *Match, unsigned int RxOpt = RX_CASE);
+int RxReplace(const char *rep, const char *Src, size_t len, RxMatchRes match, char **Dest, size_t *Dlen);
 void RxFree(RxNode *Node);
 
-#endif
+#endif // E_REGEX_H
