@@ -22,6 +22,11 @@
 
 #ifdef CONFIG_HILIT_PERL
 
+#include "c_bind.h"
+#include "o_buflist.h"
+
+#include <ctype.h>
+
 #define X_BIT     0x80     /* set if last was number, var, */
 #define X_MASK    0x7F
 #define X_NOT(state) (!((state) & X_BIT))
@@ -121,13 +126,14 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
     C = 0;
     NC = 0;
     int isEOHereDoc = 0;
+    if ((State & X_MASK) == hsPerl_HereDoc)
+    {
+        isEOHereDoc = strlen(hereDocKey) == (size_t)len &&
+            strncmp(hereDocKey, Line->Chars, len) == 0;
+        if (isEOHereDoc) State = hsPerl_Normal | (State & X_BIT);
+    }
     for(i = 0; i < Line->Count;) {
         if (*p != ' ' && *p != 9) firstnw++;
-        if ((State & X_MASK) == hsPerl_HereDoc && 0 == i)
-        {
-            isEOHereDoc = strlen(hereDocKey) == (size_t)len &&
-                strncmp(hereDocKey, Line->Chars, len) == 0;
-        }
         if (*p == '{' && inSub)
             inSub = 0;
         IF_TAB() else {
@@ -135,20 +141,23 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
             switch (State & X_MASK) {
             default:
             case hsPerl_Normal:
-                if (i == 0 && X_NOT(State) && len == 7 &&
-                    p[0] == '_' &&
-                    p[1] == '_' &&
-                    p[2] == 'E' &&
-                    p[3] == 'N' &&
-                    p[4] == 'D' &&
-                    p[5] == '_' &&
-                    p[6] == '_')
+                if (i == 0 && X_NOT(State) && len == 8 &&
+                           p[0] == '_' &&
+                           p[1] == '_' &&
+                           p[2] == 'D' &&
+                           p[3] == 'A' &&
+                           p[4] == 'T' &&
+                           p[5] == 'A' &&
+                           p[6] == '_' &&
+                           p[7] == '_')
                 {
                     State = hsPerl_Data;
                     Color = CLR_Comment;
+                hilit6:
+                    ColorNext();
                 hilit5:
                     ColorNext();
-                //hilit4:
+                hilit4:
                     ColorNext();
                 //hilit3:
                     ColorNext();
@@ -157,29 +166,64 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                 hilit:
                     ColorNext();
                     continue;
+                } else if (
+                           i == 0 && X_NOT(State) && len == 7 &&
+                           p[0] == '_' &&
+                           p[1] == '_' &&
+                           p[2] == 'E' &&
+                           p[3] == 'N' &&
+                           p[4] == 'D' &&
+                           p[5] == '_' &&
+                           p[6] == '_')
+                {
+                    State = hsPerl_Data;
+                    Color = CLR_Comment;
+
+                } else if (
+                           i == 0 && X_NOT(State) && (*p == '=') && len > 5 &&
+                           (
+                            strncmp(p+1, "begin", 5) == 0
+                           )
+                          )
+                {
+                    State = hsPerl_Docs;
+                    Color = CLR_Comment;
+                    goto hilit6;
                 } else if (
                            i == 0 && X_NOT(State) && (*p == '=') && len > 4 &&
                            (
                             strncmp(p+1, "head", 4) == 0 ||
                             strncmp(p+1, "item", 4) == 0 ||
                             strncmp(p+1, "over", 4) == 0 ||
-                            strncmp(p+1, "back", 4) == 0 ||
-                            strncmp(p+1, "pod",  3) == 0
+                            strncmp(p+1, "back", 4) == 0
                            )
                           )
                 {
                     State = hsPerl_Docs;
                     Color = CLR_Comment;
                     goto hilit5;
+                } else if (
+                           i == 0 && X_NOT(State) && (*p == '=') && len > 3 &&
+                           (
+                            strncmp(p+1, "pod",  3) == 0 ||
+                            strncmp(p+1, "for",  3) == 0 ||
+                            strncmp(p+1, "end",  3) == 0 ||
+                            0
+                           )
+                          )
+                {
+                    State = hsPerl_Docs;
+                    Color = CLR_Comment;
+                    goto hilit4;
                 } else if (isalpha(*p) || *p == '_') {
                     op = -1;
-                    
+
                     j = 0;
                     while (((i + j) < Line->Count) &&
                            (isalnum(Line->Chars[i+j]) ||
                             (Line->Chars[i + j] == '_' || Line->Chars[i + j] == '\''))
                            ) j++;
-                    if (BF->GetHilitWord(j, &Line->Chars[i], Color)) {
+                    if (BF->GetHilitWord(Color, &Line->Chars[i], j)) {
                         //Color = hcPERL_Keyword;
                         State = hsPerl_Keyword;
                         if (strncmp(p, "sub", 3) == 0)
@@ -189,7 +233,7 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                     } else {
                         int x;
                         x = i + j;
-                        while ((x < Line->Count) && 
+                        while ((x < Line->Count) &&
                                ((Line->Chars[x] == ' ') || (Line->Chars[x] == 9))) x++;
                         if ((x < Line->Count) && (Line->Chars[x] == '(')) {
                             Color = CLR_Function;
@@ -216,42 +260,42 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                     }
                     if (StateMap)
                         memset(StateMap + i, State, j);
-                    if (B) 
+                    if (B)
                         MoveMem(B, C - Pos, Width, Line->Chars + i, HILIT_CLRD(), j);
                     i += j;
                     len -= j;
                     p += j;
                     C += j;
-                    
+
                     switch (op) {
                     case opQ:
                         State = hsPerl_Quote1Op;   // q{} operator
                         Color = CLR_Punctuation;
                         continue;
-                        
+
                     case opQQ:
                     case opQW:
                     case opQX:
                         State = hsPerl_Quote1Op;   // qq{} qx{} qw{} operators
                         Color = CLR_Punctuation;
                         continue;
-                        
+
                     case opM:
                     case opQR:
                         State = hsPerl_Regexp1Op;   // m{} qr{} operator
                         Color = CLR_Punctuation;
                         continue;
-                        
+
                     case opTR:
                         State = hsPerl_Regexp2Op;   // tr{} operators
                         Color = CLR_RegexpDelim;
                         continue;
-                        
+
                     case opS:
                         State = hsPerl_Regexp2Op;   // s{}{} operator
                         Color = CLR_Punctuation;
                         continue;
-                        
+
                     default:
                         State = hsPerl_Normal;
                         continue;
@@ -290,21 +334,31 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                                          *p == '%' ||
                                          *p == '\\'))
                         ColorNext();
-                    while ((len > 0) && (isalnum(*p) || 
+                    while ((len > 0) && (isalnum(*p) ||
                                          *p == '_' ||
                                          *p == '\''))
                         ColorNext();
                     State = hsPerl_Normal | X_BIT;
                     continue;
-                } else if ((*p == '$') && (len > 1) &&
-                           ((p[1] == '$') || p[1] == '"')) {
+                } else if ((*p == '$' || *p == '*') && (len > 1) &&
+                           (p[1] == '"')) {
                     State = hsPerl_Variable;
                     Color = CLR_Variable;
                     ColorNext();
                     ColorNext();
                     State = hsPerl_Normal | X_BIT;
                     continue;
-                } else if (*p == '$' || *p == '@' || *p == '\\' || (len > 2 && (*p == '%' || *p == '*') && X_NOT(State))) {
+                } else if (*p == '*' && len > 1 && p[1] == '*') {
+                    hlState s = State;
+                    State = hsPerl_Punct;
+                    Color = CLR_Punctuation;
+                    ColorNext();
+                    ColorNext();
+                    State = s;
+                    continue;
+                } else if (*p == '$' || *p == '@' || *p == '\\' ||
+                           (len > 2 && (*p == '%' || *p == '*') && X_NOT(State))
+                          ) {
                     char var_type = *p;
                     State = hsPerl_Variable;
                     Color = CLR_Variable;
@@ -324,7 +378,7 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                         ColorNext();
                     // the following are one-character-ONLY
                     if (
-                        (var_type == '$' && strchr("_&`'+*.!/|,\\\";#%=-~:?$<>()[]", first) != NULL) ||
+                        (var_type == '$' && strchr("_&`'+*.!/|,\\\";#%=-~:?<>()[]", first) != NULL) ||
                         (var_type == '@' && strchr("_", first) != NULL)
                        )
                     {
@@ -378,29 +432,35 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
                     State = QSET(hsPerl_String2, '"');
                     Color = CLR_String;
                     goto hilit;
-                } else if (*p == '<' && len > 2 && p[1] == '<' &&
-                           (p[2] == '"' || p[2] == '\'' || p[2] == '_' || (toupper(p[2]) >= 'A' && toupper(p[2]) <= 'Z')))
-                {
-                    int hereDocKeyLen;
+                } else if (*p == '<' && len > 2 && p[1] == '<') {
                     int offset = 2;
-                    if (p[2] == '"' || p[2] == '\'')
-                        offset++;
-                    setHereDoc++;
-                    for (hereDocKeyLen = 0;
-                         hereDocKeyLen < len && (
-                                                 p[offset + hereDocKeyLen] == '_' ||
-                                                 (toupper(p[offset + hereDocKeyLen]) >= 'A' && toupper(p[offset + hereDocKeyLen]) <= 'Z')
-                                                );
-                         ++hereDocKeyLen)
+                    while (p[offset] == '"' ||
+                           p[offset] == '\'' ||
+                           isspace(p[offset]) ||
+                           0)
                     {
-                        hereDocKey[hereDocKeyLen] = p[offset + hereDocKeyLen];
+                        ++offset;
+                    }
+                    if (p[offset] == '_' || (toupper(p[offset]) >= 'A' && toupper(p[offset]) <= 'Z'))
+                    {
+                        int hereDocKeyLen;
+                        setHereDoc++;
+                        for (hereDocKeyLen = 0;
+                             hereDocKeyLen < len && (
+                                                     p[offset + hereDocKeyLen] == '_' ||
+                                                     (toupper(p[offset + hereDocKeyLen]) >= 'A' && toupper(p[offset + hereDocKeyLen]) <= 'Z')
+                                                    );
+                             ++hereDocKeyLen)
+                        {
+                            hereDocKey[hereDocKeyLen] = p[offset + hereDocKeyLen];
+                        }
+                        hereDocKey[hereDocKeyLen] = '\0';
+                        State = hsPerl_Punct;
+                        Color = CLR_Punctuation;
+                        ColorNext();
+                        State = hsPerl_Normal;
+                        continue;
                     }
-                    hereDocKey[hereDocKeyLen] = '\0';
-                    State = hsPerl_Punct;
-                    Color = CLR_Punctuation;
-                    ColorNext();
-                    State = hsPerl_Normal;
-                    continue;
                 } else if (*p == '`') {
                     State = QSET(hsPerl_StringBk, '`');
                     Color = CLR_String;
@@ -667,4 +727,4 @@ int Hilit_PERL(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line
     *ECol = C;
     return 0;
 }
-#endif
+#endif // CONFIG_HILIT_PERL
