@@ -7,11 +7,24 @@
  *
  */
 
-#include "fte.h"
+#include "o_list.h"
 
-EListPort::EListPort(EList *L, EView *V): EViewPort(V) {
-    List = L;
-    OldTopRow = OldLeftCol = OldRow = OldCount = -1;
+#include "c_color.h"
+#include "c_config.h"
+#include "i_modelview.h"
+#include "i_view.h"
+#include "o_buflist.h"
+
+#include <stdio.h>
+
+EListPort::EListPort(EList *L, EView *V) :
+    EViewPort(V),
+    List(L),
+    OldRow(-1),
+    OldTopRow(-1),
+    OldLeftCol(-1),
+    OldCount(-1)
+{
     GetPos();
 }
 
@@ -47,11 +60,11 @@ void EListPort::HandleEvent(TEvent &Event) {
     case evCommand:
         switch (Event.Msg.Command) {
         case cmVScrollUp:
-            List->ScrollDown(1);
+            List->ScrollDown(Event.Msg.Param1);
             Event.What = evNone;
             break;
         case cmVScrollDown:
-            List->ScrollUp(1);
+            List->ScrollUp(Event.Msg.Param1);
             Event.What = evNone;
             break;
         case cmVScrollPgUp:
@@ -73,11 +86,11 @@ void EListPort::HandleEvent(TEvent &Event) {
             Event.What = evNone;
             break;
         case cmHScrollLeft:
-            List->ScrollRight(1);
+            List->ScrollRight(Event.Msg.Param1);
             Event.What = evNone;
             break;
         case cmHScrollRight:
-            List->ScrollLeft(1);
+            List->ScrollLeft(Event.Msg.Param1);
             Event.What = evNone;
             break;
         case cmHScrollPgLt:
@@ -94,15 +107,18 @@ void EListPort::HandleEvent(TEvent &Event) {
             break;
         }
         break;
+#ifdef CONFIG_MOUSE
     case evMouseDown:
     case evMouseUp:
     case evMouseMove:
     case evMouseAuto:
         HandleMouse(Event);
+#endif
         break;
     }
 }
-
+ 
+#ifdef CONFIG_MOUSE
 void EListPort::HandleMouse(TEvent &Event) {
     int W, H;
     int x, y, xx, yy;
@@ -177,6 +193,7 @@ void EListPort::HandleMouse(TEvent &Event) {
         break;
     }
 }
+#endif
 
 void EListPort::UpdateView() {
     if (OldLeftCol != LeftCol || OldTopRow != TopRow || OldCount != List->Count)
@@ -262,7 +279,8 @@ void EListPort::RepaintStatus() {
         return ;
     
     View->MView->ConQuerySize(&W, &H);
-    
+
+    assert(W >= 0 && H >= 0 && W < 1000 && H < 1000);
     List->UpdateList();
     
     List->FixPos();
@@ -296,16 +314,22 @@ void EListPort::RepaintStatus() {
         View->MView->Win->ConSetCursorPos(0, Row - TopRow);
 }
 
-EList::EList(int createFlags, EModel **ARoot, const char *aTitle): EModel(createFlags, ARoot) {
-    Title = strdup(aTitle);
-    Row = TopRow = Count = LeftCol = 0;
-    NeedsUpdate = 1;
-    NeedsRedraw = 1;
-    MouseMoved = 0;
-    MouseCaptured = 0;
+EList::EList(int createFlags, EModel **ARoot, const char *aTitle) :
+    EModel(createFlags, ARoot),
+    Title(strdup(aTitle)),
+    Row(0),
+    LeftCol(0),
+    TopRow(0),
+    Count(0),
+    MouseCaptured(0),
+    MouseMoved(0),
+    NeedsUpdate(1),
+    NeedsRedraw(1)
+{
 }
 
-EList::~EList() {
+EList::~EList()
+{
     free(Title);
 }
 
@@ -323,16 +347,15 @@ EListPort *EList::GetVPort() {
     return (EListPort *)View->Port;
 }
 
-void EList::SetTitle(char *ATitle) {
-    if (Title)
-        free(Title);
+void EList::SetTitle(const char *ATitle) {
+    free(Title);
     Title = strdup(ATitle);
     if (View && View->MView)
         View->MView->RepaintStatus();
 }
     
 
-int EList::ExecCommand(int Command, ExState &State) {
+int EList::ExecCommand(ExCommands Command, ExState &State) {
     int W = 1;
     int H = 1;
     
@@ -356,7 +379,7 @@ int EList::ExecCommand(int Command, ExState &State) {
     case ExMoveFileEnd:          return MoveFileEnd();
     case ExMoveLineStart:        return MoveLineStart();
     case ExMoveLineEnd:          return MoveLineEnd();
-    case ExRescan:               RescanList();            return ErOK;
+    case ExRescan:               RescanList(); return ErOK;
     case ExActivate:             return Activate();
     case ExListMark:             return Mark();
     case ExListUnmark:           return Unmark();
@@ -364,6 +387,8 @@ int EList::ExecCommand(int Command, ExState &State) {
     case ExListMarkAll:          return MarkAll();
     case ExListUnmarkAll:        return UnmarkAll();
     case ExListToggleMarkAll:    return ToggleMarkAll();
+    default:
+	;
     }
     return EModel::ExecCommand(Command, State);
 }
@@ -665,7 +690,7 @@ int EList::Unmark() {
 }
 
 int EList::ToggleMark() {
-    if (Count > 0)
+    if (Count > 0) {
         if (IsMarked(Row)) {
             if (Unmark(Row) == 1) {
                 NeedsRedraw = 1;
@@ -677,6 +702,7 @@ int EList::ToggleMark() {
                 return ErOK;
             }
         }
+    }
     return ErFAIL;
 }
 
