@@ -7,8 +7,11 @@
  *
  */
 
-#ifndef __SDIRECT_H
-#define __SDIRECT_H
+#ifndef S_DIRECT_H
+#define S_DIRECT_H
+
+#include "sysdep.h"
+#include <sys/types.h>
 
 // error handling needs some work
 
@@ -17,16 +20,16 @@
 #define fiDIRECTORY 2
 
 class FileInfo {
-    char *name;   // minimum set of file information
+    StlString name;   // minimum set of file information
     off_t size;
     time_t mtime;
     int type;
     
 public:
-    FileInfo(char *Name, int type, off_t Size, time_t MTime);
+    FileInfo(const char *Name, int type, off_t Size, time_t MTime);
     ~FileInfo();
 
-    const char *Name() const { return name; }
+    const char *Name() const { return name.c_str(); }
     off_t Size() const { return size; }
     int Type() const { return type; }
     time_t MTime() const { return mtime; }
@@ -59,4 +62,4 @@ public:
     int FindNext(FileInfo **fi);
 };
 
-#endif
+#endif // S_DIRECT_H
